import logging
import os
import shutil
import subprocess


class FortiClient:

    def __init__(self, path):
        self.path = path
        self.bin = self.detect_binary()

    def connect(self, url, user, password):
        logging.info('Connecting to ' + url)
        proc = subprocess.Popen([self.bin, 'connect',
                                 '-h' + url,
                                 '-u' + user + ':' + password,
                                 '-m'])
        try:
            proc.communicate(timeout=2)
        except subprocess.TimeoutExpired:
            # we need to detach
            pass

    def disconnect(self):
        subprocess.Popen([self.bin, 'disconnect'])
        logging.info('Disconnected')

    def detect_binary(self):
        executable = "FortiSSLVPNclient.exe"

        bin = None

        if self.path is not None:
            if os.path.isdir(self.path):
                bin = os.path.join(self.path, executable)
            elif os.path.isfile(self.path):
                bin = self.path

            if os.path.exists(bin):
                return bin
        else:
            bin = os.path.join("C:\\Program Files (x86)\\Fortinet\\SslvpnClient", executable)
            if os.path.exists(bin):
                return bin

            bin = os.path.join("C:\\Program Files\\Fortinet\\SslvpnClient", executable)
            if os.path.exists(bin):
                return bin

            # look in FORTISSLVPN_HOME
            env_path = os.environ.get('FORTISSLVPN_HOME')
            if env_path is not None:
                bin = os.path.join(env_path, executable)
                if os.path.exists(bin):
                    return bin

            # look in PATH
            bin = shutil.which(executable)
            if bin is not None:
                return bin

        raise Exception("Could not find " + executable)



import logging
import sys

import click
from keepasshttp import keepasshttp
from fortic.fortinet import FortiClient


@click.group()
@click.option("--path", "-p", type=click.Path(exists=True, file_okay=True, dir_okay=True, readable=True),
              help="Directory or path to FortiSSLVPNclient.exe")
@click.help_option("-h", "--help")
@click.pass_context
def main(ctx, path):
    """
    Connect to Fortinet SSL VPN Gateway

    I need FortiSSLVPNclient.exe and will search in:

    \b
    - C:\\Program Files (x86)\\Fortinet\\SslvpnClient
    - C:\\Program Files\\Fortinet\\SslvpnClient
    - in -p/--path given to the command (as file or directory)
    - in FORTISSLVPN_HOME environment variable
    - availability in PATH variable
    """
    ctx.obj = path
    pass


@click.command(help="Endpoint address")
@click.argument("url")
@click.pass_context
def connect(ctx, url):
    try:
        client = FortiClient(ctx.obj)
        creds = get_credentials(url)
        client.connect(url, creds.login, creds.password)
        sys.exit(0)
    except Exception as e:
        logging.error(e)
        sys.exit(1)


@click.command(help="Disconnect")
@click.pass_context
def disconnect(ctx):
    try:
        client = FortiClient(ctx.obj)
        client.disconnect()
        sys.exit(0)
    except Exception as e:
        logging.error(e)
        sys.exit(1)


main.add_command(connect)
main.add_command(disconnect)


def get_credentials(url):
    logging.info("Retrieve credentials for '" + url + "'")
    credentials = keepasshttp.get(url)
    if credentials is None:
        raise Exception(
            "KeePass entry for '" + url + "' not found! Please add an entry with '" + url + "' as name or url")

    logging.info(f"Credentials found (User: {credentials.login})")
    return credentials


if __name__ == '__main__':
    logging.basicConfig(format="[%(levelname)-5s] %(message)s", stream=sys.stdout, level=logging.DEBUG)
    main()



from setuptools import setup

with open('README.md') as readme_file:
    README = readme_file.read()

setup(
    name='fortic',
    packages=['fortic'],
    version='0.1',
    license='apache-2.0',
    description='Fortinet SSLVPN command line interface',
    author='Juergen Schmid',
    author_email='jur.schmid@gmail.com',
    url='https://github.com/hacki11/fortic',
    keywords=['vpn', 'fortinet', 'sslvpn', 'cli'],
    install_requires=[
        'click',
        'keepasshttp'
    ],
    entry_points={
        'console_scripts': ['fortic=fortic.fortic:main'],
    },
    classifiers=[
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python :: 3',
        'Operating System :: Microsoft :: Windows'
    ],
)